@@ -1,7 +1,7 @@
 /* mpfr_erf -- error function of a floating-point number
 
-Copyright 2001, 2003-2022 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 2001, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+Contributed by the AriC and Caramel projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -17,19 +17,20 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #define MPFR_NEED_LONGLONG_H
 #include "mpfr-impl.h"
 
+#define EXP1 2.71828182845904523536 /* exp(1) */
+
 static int mpfr_erf_0 (mpfr_ptr, mpfr_srcptr, double, mpfr_rnd_t);
 
 int
 mpfr_erf (mpfr_ptr y, mpfr_srcptr x, mpfr_rnd_t rnd_mode)
 {
   mpfr_t xf;
-  mp_limb_t xf_limb[(53 - 1) / GMP_NUMB_BITS + 1];
   int inex, large;
   MPFR_SAVE_EXPO_DECL (expo);
 
@@ -78,7 +79,7 @@ mpfr_erf (mpfr_ptr y, mpfr_srcptr x, mpfr_rnd_t rnd_mode)
       mpfr_init2 (l, MPFR_PREC(y) + 17);
       mpfr_init2 (h, MPFR_PREC(y) + 17);
       /* first compute l */
-      mpfr_sqr (l, x, MPFR_RNDU);
+      mpfr_mul (l, x, x, MPFR_RNDU);
       mpfr_div_ui (l, l, 3, MPFR_RNDU); /* upper bound on x^2/3 */
       mpfr_ui_sub (l, 1, l, MPFR_RNDZ); /* lower bound on 1 - x^2/3 */
       mpfr_const_pi (h, MPFR_RNDU); /* upper bound of Pi */
@@ -109,11 +110,13 @@ mpfr_erf (mpfr_ptr y, mpfr_srcptr x, mpfr_rnd_t rnd_mode)
          the error term x^2/3 is not that small. */
     }
 
-  MPFR_TMP_INIT1(xf_limb, xf, 53);
-  mpfr_div (xf, x, __gmpfr_const_log2_RNDU, MPFR_RNDZ); /* round to zero
-                        ensures we get a lower bound of |x/log(2)| */
+  mpfr_init2 (xf, 53);
+  mpfr_const_log2 (xf, MPFR_RNDU);
+  mpfr_div (xf, x, xf, MPFR_RNDZ); /* round to zero ensures we get a lower
+                                     bound of |x/log(2)| */
   mpfr_mul (xf, xf, x, MPFR_RNDZ);
   large = mpfr_cmp_ui (xf, MPFR_PREC (y) + 1) > 0;
+  mpfr_clear (xf);
 
   /* when x goes to infinity, we have erf(x) = 1 - 1/sqrt(Pi)/exp(x^2)/x + ...
      and |erf(x) - 1| <= exp(-x^2) is true for any x >= 0, thus if
@@ -154,21 +157,15 @@ mpfr_erf (mpfr_ptr y, mpfr_srcptr x, mpfr_rnd_t rnd_mode)
 static double
 mul_2exp (double x, mpfr_exp_t e)
 {
-  /* Most of the times, the argument is negative */
-  if (MPFR_UNLIKELY (e > 0))
+  if (e > 0)
     {
       while (e--)
         x *= 2.0;
     }
   else
     {
-      while (e <= -16)
-        {
-          x *= (1.0 / 65536.0);
-          e += 16;
-        }
       while (e++)
-        x *= 0.5;
+        x /= 2.0;
     }
 
   return x;
@@ -186,10 +183,11 @@ mpfr_erf_0 (mpfr_ptr res, mpfr_srcptr x, double xf2, mpfr_rnd_t rnd_mode)
 {
   mpfr_prec_t n, m;
   mpfr_exp_t nuk, sigmak;
+  double tauk;
   mpfr_t y, s, t, u;
   unsigned int k;
+  int log2tauk;
   int inex;
-  MPFR_GROUP_DECL (group);
   MPFR_ZIV_DECL (loop);
 
   n = MPFR_PREC (res); /* target precision */
@@ -197,19 +195,18 @@ mpfr_erf_0 (mpfr_ptr res, mpfr_srcptr x, double xf2, mpfr_rnd_t rnd_mode)
   /* initial working precision */
   m = n + (mpfr_prec_t) (xf2 / LOG2) + 8 + MPFR_INT_CEIL_LOG2 (n);
 
-  MPFR_GROUP_INIT_4(group, m, y, s, t, u);
+  mpfr_init2 (y, m);
+  mpfr_init2 (s, m);
+  mpfr_init2 (t, m);
+  mpfr_init2 (u, m);
 
   MPFR_ZIV_INIT (loop, m);
   for (;;)
     {
-      mpfr_t tauk;
-      mpfr_exp_t log2tauk;
-
-      mpfr_sqr (y, x, MPFR_RNDU); /* err <= 1 ulp */
+      mpfr_mul (y, x, x, MPFR_RNDU); /* err <= 1 ulp */
       mpfr_set_ui (s, 1, MPFR_RNDN);
       mpfr_set_ui (t, 1, MPFR_RNDN);
-      mpfr_init2 (tauk, 53);
-      mpfr_set_ui (tauk, 0, MPFR_RNDU);
+      tauk = 0.0;
 
       for (k = 1; ; k++)
         {
@@ -224,13 +221,12 @@ mpfr_erf_0 (mpfr_ptr res, mpfr_srcptr x, double xf2, mpfr_rnd_t rnd_mode)
           sigmak -= MPFR_GET_EXP(s);
           nuk = MPFR_GET_EXP(u) - MPFR_GET_EXP(s);
 
-          if ((nuk < - (mpfr_exp_t) m) && (k >= xf2))
+          if ((nuk < - (mpfr_exp_t) m) && ((double) k >= xf2))
             break;
 
           /* tauk <- 1/2 + tauk * 2^sigmak + (1+8k)*2^nuk */
-          mpfr_mul_2si (tauk, tauk, sigmak, MPFR_RNDU);
-          mpfr_add_d (tauk, tauk, 0.5 + mul_2exp (1.0 + 8.0 * (double) k, nuk),
-                      MPFR_RNDU);
+          tauk = 0.5 + mul_2exp (tauk, sigmak)
+            + mul_2exp (1.0 + 8.0 * (double) k, nuk);
         }
 
       mpfr_mul (s, x, s, MPFR_RNDU);
@@ -239,27 +235,28 @@ mpfr_erf_0 (mpfr_ptr res, mpfr_srcptr x, double xf2, mpfr_rnd_t rnd_mode)
       mpfr_const_pi (t, MPFR_RNDZ);
       mpfr_sqrt (t, t, MPFR_RNDZ);
       mpfr_div (s, s, t, MPFR_RNDN);
-      /* tauk = 4 * tauk + 11: final ulp-error on s */
-      mpfr_mul_2ui (tauk, tauk, 2, MPFR_RNDU);
-      mpfr_add_ui (tauk, tauk, 11, MPFR_RNDU);
-      /* In practice, one should not get an infinity, as it would require
-         a huge precision and a lot of time, but just in case... */
-      MPFR_ASSERTN (!MPFR_IS_INF (tauk));
-      log2tauk = MPFR_GET_EXP (tauk);
-      mpfr_clear (tauk);
+      tauk = 4.0 * tauk + 11.0; /* final ulp-error on s */
+      log2tauk = __gmpfr_ceil_log2 (tauk);
 
       if (MPFR_LIKELY (MPFR_CAN_ROUND (s, m - log2tauk, n, rnd_mode)))
         break;
 
       /* Actualisation of the precision */
       MPFR_ZIV_NEXT (loop, m);
-      MPFR_GROUP_REPREC_4 (group, m, y, s, t, u);
+      mpfr_set_prec (y, m);
+      mpfr_set_prec (s, m);
+      mpfr_set_prec (t, m);
+      mpfr_set_prec (u, m);
+
     }
   MPFR_ZIV_FREE (loop);
 
   inex = mpfr_set (res, s, rnd_mode);
 
-  MPFR_GROUP_CLEAR (group);
+  mpfr_clear (y);
+  mpfr_clear (t);
+  mpfr_clear (u);
+  mpfr_clear (s);
 
   return inex;
 }
