@@ -1,7 +1,7 @@
 /* mpfr_set_default_prec, mpfr_get_default_prec -- set/get default precision
 
-Copyright 1999-2001, 2004-2022 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 1999, 2000, 2001, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+Contributed by the AriC and Caramel projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -17,19 +17,19 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include "mpfr-impl.h"
 
 /* default is IEEE double precision, i.e. 53 bits */
-MPFR_THREAD_VAR (mpfr_prec_t, __gmpfr_default_fp_bit_precision,
-                 IEEE_DBL_MANT_DIG)
+mpfr_prec_t MPFR_THREAD_ATTR __gmpfr_default_fp_bit_precision \
+  = IEEE_DBL_MANT_DIG;
 
 void
 mpfr_set_default_prec (mpfr_prec_t prec)
 {
-  MPFR_ASSERTN (MPFR_PREC_COND (prec));
+  MPFR_ASSERTN (prec >= MPFR_PREC_MIN && prec <= MPFR_PREC_MAX);
   __gmpfr_default_fp_bit_precision = prec;
 }
 
