@@ -1,7 +1,7 @@
 /* mpfr_get_version -- MPFR version
 
-Copyright 2004-2025 Free Software Foundation, Inc.
-Contributed by the Pascaline and Caramba projects, INRIA.
+Copyright 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+Contributed by the AriC and Caramel projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,13 +16,14 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.
-If not, see <https://www.gnu.org/licenses/>. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
+http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include "mpfr-impl.h"
 
 const char *
 mpfr_get_version (void)
 {
-  return "4.2.2";
+  return "3.1.1-p2";
 }
