@@ -1,7 +1,7 @@
 /* tinternals -- Test for internals.
 
-Copyright 2005-2020 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+Contributed by the AriC and Caramel projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -17,9 +17,12 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
+#include <stdio.h>
+#include <stdlib.h>
+
 #define MPFR_NEED_LONGLONG_H
 #include "mpfr-test.h"
 
@@ -67,7 +70,7 @@ test_round_near_x (void)
            mpfr_neg (x, x, MPFR_RNDN), p++, neg++)
         for (err = 2; err <= 6; err++)
           for (dir = 0; dir <= 1; dir++)
-            RND_LOOP_NO_RNDF (r)
+            RND_LOOP(r)
               {
                 inex = mpfr_round_near_x (y, x, err, dir, (mpfr_rnd_t) r);
 
@@ -144,7 +147,11 @@ main (int argc, char **argv)
 {
   tests_start_mpfr ();
 
+  /* The tested function and macro exist in MPFR 2.2.0, but with a
+     different (incorrect, but with no effect in 2.2.0) behavior. */
+#if MPFR_VERSION >= MPFR_VERSION_NUM(2,3,0)
   test_int_ceil_log2 ();
+#endif
 
   test_round_near_x ();
   test_set_prec_raw ();
